@@ -92,8 +92,20 @@ def params_to_string(params_num, units=None, precision=2):
             return str(params_num)
 
 
+def accumulate_flops(self):
+    if is_supported_instance(self):
+        return self.__flops__
+    else:
+        sum = 0
+        for m in self.children():
+            sum += m.accumulate_flops()
+        return sum
+
+
 def print_model_with_flops(model, total_flops, total_params, units='GMac',
                            precision=3, ost=sys.stdout):
+    if total_flops < 1:
+        total_flops = 1
 
     def accumulate_params(self):
         if is_supported_instance(self):
@@ -104,18 +116,9 @@ def print_model_with_flops(model, total_flops, total_params, units='GMac',
                 sum += m.accumulate_params()
             return sum
 
-    def accumulate_flops(self):
-        if is_supported_instance(self):
-            return self.__flops__ / model.__batch_counter__
-        else:
-            sum = 0
-            for m in self.children():
-                sum += m.accumulate_flops()
-            return sum
-
     def flops_repr(self):
         accumulated_params_num = self.accumulate_params()
-        accumulated_flops_cost = self.accumulate_flops()
+        accumulated_flops_cost = self.accumulate_flops() / model.__batch_counter__
         return ', '.join([params_to_string(accumulated_params_num,
                                            units='M', precision=precision),
                           '{:.3%} Params'.format(accumulated_params_num / total_params),
@@ -173,14 +176,17 @@ def compute_average_flops_cost(self):
 
     """
 
-    batches_count = self.__batch_counter__
-    flops_sum = 0
-    params_sum = 0
-    for module in self.modules():
-        if is_supported_instance(module):
-            flops_sum += module.__flops__
+    for m in self.modules():
+        m.accumulate_flops = accumulate_flops.__get__(m)
+
+    flops_sum = self.accumulate_flops()
+
+    for m in self.modules():
+        if hasattr(m, 'accumulate_flops'):
+            del m.accumulate_flops
+
     params_sum = get_model_parameters_number(self)
-    return flops_sum / batches_count, params_sum
+    return flops_sum / self.__batch_counter__, params_sum
 
 
 def start_flops_count(self, **kwargs):
@@ -406,6 +412,43 @@ def rnn_cell_flops_counter_hook(rnn_cell_module, input, output):
     rnn_cell_module.__flops__ += int(flops)
 
 
+def multihead_attention_counter_hook(multihead_attention_module, input, output):
+    flops = 0
+    q, k, v = input
+    batch_size = q.shape[1]
+
+    num_heads = multihead_attention_module.num_heads
+    embed_dim = multihead_attention_module.embed_dim
+    kdim = multihead_attention_module.kdim
+    vdim = multihead_attention_module.vdim
+    if kdim is None:
+        kdim = embed_dim
+    if vdim is None:
+        vdim = embed_dim
+
+    # initial projections
+    flops = q.shape[0] * q.shape[2] * embed_dim + \
+        k.shape[0] * k.shape[2] * kdim + \
+        v.shape[0] * v.shape[2] * vdim
+    if multihead_attention_module.in_proj_bias is not None:
+        flops += (q.shape[0] + k.shape[0] + v.shape[0]) * embed_dim
+
+    # attention heads: scale, matmul, softmax, matmul
+    head_dim = embed_dim // num_heads
+    head_flops = q.shape[0] * head_dim + \
+        head_dim * q.shape[0] * k.shape[0] + \
+        q.shape[0] * k.shape[0] + \
+        q.shape[0] * k.shape[0] * head_dim
+
+    flops += num_heads * head_flops
+
+    # final projection, bias is always enabled
+    flops += q.shape[0] * embed_dim * (embed_dim + 1)
+
+    flops *= batch_size
+    multihead_attention_module.__flops__ += int(flops)
+
+
 def add_batch_counter_variables_or_reset(module):
 
     module.__batch_counter__ = 0
@@ -479,7 +522,8 @@ MODULES_MAPPING = {
     nn.LSTM: rnn_flops_counter_hook,
     nn.RNNCell: rnn_cell_flops_counter_hook,
     nn.LSTMCell: rnn_cell_flops_counter_hook,
-    nn.GRUCell: rnn_cell_flops_counter_hook
+    nn.GRUCell: rnn_cell_flops_counter_hook,
+    nn.MultiheadAttention: multihead_attention_counter_hook
 }
 
 
